/*
 * Modified by Akihiro Tominaga. (tomy@sfc.wide.ad.jp)
 */
/*
 * Copyright (c) 1990, 1992 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code distributions
 * retain the above copyright notice and this paragraph in its entirety, (2)
 * distributions including binary code include the above copyright notice and
 * this paragraph in its entirety in the documentation or other materials
 * provided with the distribution, and (3) all advertising materials mentioning
 * features or use of this software display the following acknowledgement:
 * ``This product includes software developed by the University of California,
 * Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
 * the University nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
#ifdef lint
static char rcsid[] =
    "@(#) $Header: myetheraddr.c,v 1.6 92/03/17 18:43:30 leres Exp $ (LBL)";
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/param.h>
#include <sys/time.h>
#include <sys/proc.h>

#include <sys/socket.h>
#include <net/if.h>

#if defined(__FreeBSD__) || defined(__bsdi__)
#if defined(__FreeBSD__)
#include <osreldate.h>
#if (__FreeBSD_version == 199702) || (__FreeBSD_version >= 300000)
#include <net/if_var.h>
#endif
#endif /* defined(__FreeBSD__) */
#include <sys/sysctl.h>
#include <net/route.h>
#include <net/if_dl.h>

#else /* defined(__FreeBSD__) || defined(__bsdi__) */

#include <nlist.h>
#include <kvm.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <netinet/in_var.h>
#include <netinet/if_ether.h>
#include <arpa/inet.h>

#endif /* defined(__FreeBSD__) || defined(__bsdi__) */

/* Determine if "bits" is set in "flag" */
#define ALLSET(flag, bits) (((flag) & (bits)) == (bits))

#if defined(__FreeBSD__) || defined(__bsdi__)

int
getmac(ifname, result)
  char *ifname;
  char *result;
{
  int flags;
  struct  if_msghdr *ifm, *nextifm;
  struct  ifa_msghdr *ifam;
  struct  sockaddr_dl *sdl;
  char    *buf, *lim, *next;
  size_t needed;
  int mib[6];

  mib[0] = CTL_NET;
  mib[1] = PF_ROUTE;
  mib[2] = 0;
  mib[3] = AF_INET;
  mib[4] = NET_RT_IFLIST;
  mib[5] = 0;

  if (sysctl(mib, 6, NULL, &needed, NULL, 0) < 0) {
    syslog(LOG_ERR, "iflist-sysctl-estimate error in getmac(): %m");
    return(-1);
  }     
  if ((buf = malloc(needed)) == NULL) {
    syslog(LOG_ERR, "malloc error in getmac(): %m");
    return(-1);
  }
  if (sysctl(mib, 6, buf, &needed, NULL, 0) < 0) {
    syslog(LOG_ERR, "iflist-sysctl error in getmac(): %m");                
    return(-1);
  }
  lim = buf + needed;

  next = buf;
  while (next < lim) {

    ifm = (struct if_msghdr *)next;
                
    if (ifm->ifm_type == RTM_IFINFO) {
      sdl = (struct sockaddr_dl *)(ifm + 1);
      flags = ifm->ifm_flags;
    } else {
      syslog(LOG_ERR, "out of sync parsing NET_RT_IFLIST");
      syslog(LOG_ERR, "expected %d, got %d", RTM_IFINFO, ifm->ifm_type);
      syslog(LOG_ERR, "msglen = %d", ifm->ifm_msglen);
      syslog(LOG_ERR, "buf:%p, next:%p, lim:%p", buf, next, lim);
      return (-1);
    }

    next += ifm->ifm_msglen;
    ifam = NULL;
    while (next < lim) {

      nextifm = (struct if_msghdr *)next;

      if (nextifm->ifm_type != RTM_NEWADDR)
        break;

      if (ifam == NULL)
        ifam = (struct ifa_msghdr *)nextifm;

        next += nextifm->ifm_msglen;
    }

    /* Only look at configured, broadcast interfaces */
    if (!ALLSET(flags, IFF_UP | IFF_BROADCAST))
      continue;

    /* Only look at the specified interface */
    if (strlen(ifname) != sdl->sdl_nlen)
      continue; /* not same len */
    if (strncmp(ifname, sdl->sdl_data, sdl->sdl_nlen) != 0)
      continue; /* not same name */

    /* found */
    bcopy(LLADDR(sdl), result, 6);
    free(buf);
    return(0);
  }
  /* not found */
  free(buf);
  errno = 0;
  syslog(LOG_ERR, "getmac() failed");
  return(-1);
}

#else /* defined(__FreeBSD__) || defined(__bsdi__) */

static struct nlist nl[] = {
#define N_IFNET 0
	{ "_ifnet" },
	{ 0 }
};


int
getmac(ifname, result)
  char *ifname;
  char *result;
{
  register struct ifnet *ifp = NULL;
  register struct arpcom *ac = NULL;
  register u_char *ep = NULL;
  struct arpcom arpcom;
  register struct ifaddr *ifa = NULL;
  register struct in_ifaddr *in = NULL;
  union {
    struct ifaddr ifa;
    struct in_ifaddr in;
  } ifaddr;
  u_long addr = 0;
  char name[16];
  char fullname[16];
  static kvm_t *kd = NULL;

  bzero(&arpcom, sizeof(arpcom));
  bzero(&ifaddr, sizeof(ifaddr));
  bzero(name, sizeof(name));
  bzero(fullname, sizeof(fullname));

  /* Open kernel memory for reading */
  kd = kvm_open(0, 0, 0, O_RDONLY, "kvm_open");
  if (kd == 0) {
    syslog(LOG_ERR, "kvm_open() error in getmac(): %m");
    return(-1);
  }

  /* Fetch namelist */
  if (kvm_nlist(kd, nl) != 0) {
    syslog(LOG_ERR, "bad nlist in getmac(): %m");
    kvm_close(kd);
    return(-1);
  }

  ac = &arpcom;
  ifp = &arpcom.ac_if;
  ep = arpcom.ac_enaddr.ether_addr_octet;
  ifa = &ifaddr.ifa;
  in = &ifaddr.in;

  if (kvm_read(kd, nl[N_IFNET].n_value, (char *)&addr, sizeof(addr)) !=
               sizeof(addr)) {
    syslog(LOG_ERR, "kvm_read() error in getmac(): %m");
    kvm_close(kd);
    return(-1);
  }
  for ( ; addr; addr = (u_long)ifp->if_next) {
    if (kvm_read(kd, addr, (char *)ac, sizeof(*ac)) != sizeof(*ac)) {
      syslog(LOG_ERR, "kvm_read() error in getmac(): %m");
      kvm_close(kd);
      return(-1);
    }
    /* Only look at configured, broadcast interfaces */
    if (!ALLSET(ifp->if_flags, IFF_UP | IFF_BROADCAST))
      continue;

    /* Only look at the specified interface */
    if (kvm_read(kd, (u_long)ifp->if_name, (char *)name, sizeof(name)) !=
                 sizeof(name)) {
      syslog(LOG_ERR, "kvm_read() error in getmac(): %m");
      kvm_close(kd);
      return(-1);
    }
    name[sizeof(name) - 1] = '\0';
    sprintf(fullname, "%s%d", name, ifp->if_unit);
    if (strcmp(fullname, ifname) != 0)
      continue;

    kvm_close(kd);
    bcopy(ep, result, 6);
    return(0);
  }

  kvm_close(kd);
  errno = 0;
  syslog(LOG_ERR, "getmac() is failed");
  return(-1);
}
#endif /* defined(__FreeBSD__) || defined(__bsdi__) */
